"""End-to-end tests for file upload and processing operations."""

import time
from pathlib import Path

import pytest


class TestFileOperations:
    """Test file upload, processing, and integration with comprehensive verification."""

    def test_file_upload_text_file(
        self, api_client, resource_tracker, test_data_room, sample_text_file, api_credentials
    ):
        """Test uploading a text file and verify processing completion."""
        import uuid

        data_room_id = test_data_room["id"]
        session_id = str(uuid.uuid4())

        # Upload file
        upload_response = api_client.core.upload_file(str(sample_text_file), data_room_id, session_id)

        assert isinstance(upload_response, dict)

        # Extract upload ID from response (handle nested structure)
        upload_id = (
            upload_response.get("upload_id")
            or upload_response.get("file_id")
            or upload_response.get("id")
            or upload_response.get("body", {}).get("_id")
            or upload_response.get("body", {}).get("id")
        )

        assert upload_id, f"No upload ID found in response: {upload_response}"

        if upload_id:
            resource_tracker.track_file(upload_id)

        # Give upload time to process (simplified approach)
        time.sleep(5)

        # GET verification: Check file appears in data room
        room_details = api_client.core.get_data_room(data_room_id)
        assert isinstance(room_details, dict)

        # Should have files or documents section
        if "files" in room_details:
            files = room_details["files"]
            assert len(files) > 0, "No files found in data room after upload"

            # Verify our file is listed
            file_names = [f.get("name", "") for f in files if isinstance(f, dict)]
            assert any("sample.txt" in name for name in file_names)

    def test_file_upload_json_file(
        self, api_client, resource_tracker, test_data_room, sample_json_file, api_credentials
    ):
        """Test uploading a JSON file and verify content processing."""
        import uuid

        data_room_id = test_data_room["id"]
        session_id = str(uuid.uuid4())

        # Upload JSON file
        upload_response = api_client.core.upload_file(str(sample_json_file), data_room_id, session_id)

        assert isinstance(upload_response, dict)

        # Extract upload ID from response (handle nested structure)
        upload_id = (
            upload_response.get("upload_id")
            or upload_response.get("file_id")
            or upload_response.get("id")
            or upload_response.get("body", {}).get("_id")
            or upload_response.get("body", {}).get("id")
        )

        assert upload_id, f"No upload ID found in response: {upload_response}"

        if upload_id:
            resource_tracker.track_file(upload_id)

        # Give upload time to process (simplified approach)
        time.sleep(5)

        # Test chat integration with uploaded JSON
        chat_response = api_client.chat(
            "What data is in the uploaded JSON file? Mention specific values.", convo_id=data_room_id
        )

        assert isinstance(chat_response, (str, dict))

        if isinstance(chat_response, str):
            response_text = chat_response.lower()
        else:
            response_text = (
                chat_response.get("response") or chat_response.get("answer") or chat_response.get("content") or ""
            ).lower()

        # Should reference content from JSON file
        json_content_indicators = ["1000", "5000", "0.15", "test_data", "metrics"]
        assert any(
            indicator in response_text for indicator in json_content_indicators
        ), f"Response doesn't reference JSON content: {response_text}"

    def test_file_upload_markdown_file(
        self, api_client, resource_tracker, test_data_room, sample_md_file, api_credentials
    ):
        """Test uploading a Markdown file and verify structure processing."""
        import uuid

        data_room_id = test_data_room["id"]
        session_id = str(uuid.uuid4())

        # Upload Markdown file
        upload_response = api_client.core.upload_file(str(sample_md_file), data_room_id, session_id)

        assert isinstance(upload_response, dict)

        # Extract upload ID from response (handle nested structure)
        upload_id = (
            upload_response.get("upload_id")
            or upload_response.get("file_id")
            or upload_response.get("id")
            or upload_response.get("body", {}).get("_id")
            or upload_response.get("body", {}).get("id")
        )

        assert upload_id, f"No upload ID found in response: {upload_response}"

        if upload_id:
            resource_tracker.track_file(upload_id)

        # Give upload time to process (simplified approach)
        time.sleep(5)

        # Test that markdown structure is understood
        chat_response = api_client.chat(
            "What sections and headers are in the uploaded markdown document?", convo_id=data_room_id
        )

        assert isinstance(chat_response, (str, dict))

        if isinstance(chat_response, str):
            response_text = chat_response.lower()
        else:
            response_text = (
                chat_response.get("response") or chat_response.get("answer") or chat_response.get("content") or ""
            ).lower()

        # Should reference markdown structure
        md_structure = ["overview", "features", "test data", "code example", "conclusion"]
        assert any(
            section in response_text for section in md_structure
        ), f"Response doesn't reference markdown structure: {response_text}"

    def test_multiple_file_upload_same_room(
        self, api_client, resource_tracker, test_data_room, sample_text_file, sample_json_file, api_credentials
    ):
        """Test uploading multiple files to same data room and verify both are accessible."""
        import uuid

        data_room_id = test_data_room["id"]
        session_id = str(uuid.uuid4())
        uploaded_files = []

        # Upload multiple files
        for file_path in [sample_text_file, sample_json_file]:
            upload_response = api_client.core.upload_file(str(file_path), data_room_id, session_id)

            assert isinstance(upload_response, dict)

            # Extract upload ID from response (handle nested structure)
            upload_id = (
                upload_response.get("upload_id")
                or upload_response.get("file_id")
                or upload_response.get("id")
                or upload_response.get("body", {}).get("_id")
                or upload_response.get("body", {}).get("id")
            )

            assert upload_id, f"No upload ID found in response: {upload_response}"

            if upload_id:
                resource_tracker.track_file(upload_id)
                uploaded_files.append(upload_id)

        # Give uploads time to process (simplified approach)
        time.sleep(8)  # Longer for multiple files

        # GET verification: Check both files in data room
        room_details = api_client.core.get_data_room(data_room_id)
        if isinstance(room_details, dict) and "files" in room_details:
            files = room_details["files"]
            file_names = [f.get("name", "") for f in files if isinstance(f, dict)]

            assert any("sample.txt" in name for name in file_names), "Text file not found"
            assert any("sample.json" in name for name in file_names), "JSON file not found"

        # Test chat can access both files
        chat_response = api_client.chat(
            "I've uploaded both a text file and a JSON file. Can you tell me about both?", convo_id=data_room_id
        )

        assert isinstance(chat_response, (str, dict))

        if isinstance(chat_response, str):
            response_text = chat_response.lower()
        else:
            response_text = (
                chat_response.get("response") or chat_response.get("answer") or chat_response.get("content") or ""
            ).lower()

        # Should reference both file types
        assert any(word in response_text for word in ["text", "txt"]), "No text file reference"
        assert any(word in response_text for word in ["json", "data"]), "No JSON file reference"

    def test_file_upload_error_handling(self, api_client, test_data_room, api_credentials):
        """Test file upload error scenarios."""
        import uuid

        data_room_id = test_data_room["id"]
        session_id = str(uuid.uuid4())

        # Test non-existent file
        try:
            upload_response = api_client.core.upload_file("/non/existent/file.txt", data_room_id, session_id)
            # Should handle error gracefully
            assert isinstance(upload_response, dict)
            assert "error" in upload_response or "status" in upload_response
        except FileNotFoundError:
            # FileNotFoundError is also acceptable
            pass

    def test_file_upload_status_tracking(
        self, api_client, resource_tracker, test_data_room, sample_text_file, api_credentials
    ):
        """Test that file upload is successful."""
        import uuid

        data_room_id = test_data_room["id"]
        session_id = str(uuid.uuid4())

        # Upload file
        upload_response = api_client.core.upload_file(str(sample_text_file), data_room_id, session_id)

        assert isinstance(upload_response, dict)

        # Extract upload ID from response (handle nested structure)
        upload_id = (
            upload_response.get("upload_id")
            or upload_response.get("file_id")
            or upload_response.get("id")
            or upload_response.get("body", {}).get("_id")
            or upload_response.get("body", {}).get("id")
        )

        assert upload_id, f"No upload ID found in response: {upload_response}"

        if upload_id:
            resource_tracker.track_file(upload_id)

        # Give upload time to process
        time.sleep(5)

        # Verify upload was successful by checking data room
        room_details = api_client.core.get_data_room(data_room_id)
        # Just verify we got a response - upload was successful if no errors occurred
        assert isinstance(room_details, dict)

    def test_file_upload_with_immediate_chat(self, api_client, resource_tracker, sample_text_file):
        """Test uploading file and immediately using it in chat (via convenience method)."""
        # Use the convenience method that handles upload + chat
        chat_response = api_client.chat("Summarize the content of this file", file_path=str(sample_text_file))

        assert isinstance(chat_response, (str, dict))

        if isinstance(chat_response, str):
            response_text = chat_response.lower()
        else:
            response_text = (
                chat_response.get("response") or chat_response.get("answer") or chat_response.get("content") or ""
            ).lower()

        # Should reference file content
        file_content_keywords = ["technology", "ai", "data processing", "api integration", "testing"]
        assert any(
            keyword in response_text for keyword in file_content_keywords
        ), f"Response doesn't reference file content: {response_text}"

        # Track conversation for cleanup
        if "conversation_id" in chat_response:
            resource_tracker.track_conversation(chat_response["conversation_id"])
