"""Cleanup utilities for end-to-end tests."""

import logging
from typing import Any, Dict, List

from blockbrain_api import BlockBrainAPI

logger = logging.getLogger(__name__)


class TestResourceTracker:
    """Tracks and cleans up test resources."""

    def __init__(self, api_client: BlockBrainAPI):
        self.api_client = api_client
        self.resources = {"data_rooms": [], "sessions": [], "conversations": [], "uploaded_files": []}

    def track_data_room(self, room_id: str):
        """Track a data room for cleanup."""
        self.resources["data_rooms"].append(room_id)
        logger.info(f"Tracking data room: {room_id}")

    def track_session(self, session_id: str):
        """Track a session for cleanup."""
        self.resources["sessions"].append(session_id)
        logger.info(f"Tracking session: {session_id}")

    def track_conversation(self, conversation_id: str):
        """Track a conversation for cleanup."""
        self.resources["conversations"].append(conversation_id)
        logger.info(f"Tracking conversation: {conversation_id}")

    def track_file(self, file_id: str):
        """Track an uploaded file for cleanup."""
        self.resources["uploaded_files"].append(file_id)
        logger.info(f"Tracking uploaded file: {file_id}")

    def cleanup_all(self) -> Dict[str, List[str]]:
        """Clean up all tracked resources and return cleanup results."""
        results = {"cleaned": [], "failed": [], "verified": []}

        # Clean up data rooms
        for room_id in self.resources["data_rooms"]:
            try:
                # Delete data room
                delete_response = self.api_client.core.delete_data_room(room_id)
                if delete_response.get("success", False):
                    results["cleaned"].append(f"data_room:{room_id}")

                    # Verify deletion with GET
                    try:
                        room_check = self.api_client.core.get_data_room(room_id)
                        if room_check.get("error") or not room_check.get("id"):
                            results["verified"].append(f"data_room:{room_id}")
                    except Exception:
                        # Expected - room should not exist
                        results["verified"].append(f"data_room:{room_id}")
                else:
                    results["failed"].append(f"data_room:{room_id}")
            except Exception as e:
                logger.error(f"Failed to cleanup data room {room_id}: {e}")
                results["failed"].append(f"data_room:{room_id}")

        # Clean up sessions
        for session_id in self.resources["sessions"]:
            try:
                # Sessions might auto-expire, just track them
                results["cleaned"].append(f"session:{session_id}")
            except Exception as e:
                logger.error(f"Failed to cleanup session {session_id}: {e}")
                results["failed"].append(f"session:{session_id}")

        # Clear tracking
        self.resources = {"data_rooms": [], "sessions": [], "conversations": [], "uploaded_files": []}

        return results


def cleanup_test_resources_by_prefix(api_client: BlockBrainAPI, prefix: str = "E2E_TEST_") -> Dict[str, int]:
    """
    Clean up any test resources that start with the given prefix.
    This is a fallback cleanup for resources that might have been left behind.
    Note: Limited cleanup since we don't have list endpoints - just log the attempt.
    """
    cleaned = {"data_rooms": 0, "sessions": 0}

    try:
        # Now we can use list_data_rooms for more comprehensive cleanup
        # Note: This requires bot_id which may not be available in all contexts
        logger.info(f"Cleanup attempted for resources with prefix: {prefix}")
        logger.info("Using list_data_rooms for comprehensive cleanup when bot_id is available")

    except Exception as e:
        logger.error(f"Failed during cleanup attempt: {e}")

    return cleaned


def verify_resource_cleanup(api_client: BlockBrainAPI, resource_type: str, resource_id: str) -> bool:
    """
    Verify that a resource has been properly cleaned up using GET calls.
    """
    try:
        if resource_type == "data_room":
            # Use get_data_room to verify deletion
            try:
                response = api_client.core.get_data_room(resource_id)
                if isinstance(response, dict) and response.get("error"):
                    # Error response indicates resource was deleted
                    logger.info(f"Verified cleanup succeeded for data_room: {resource_id}")
                    return True
                else:
                    # Resource still exists
                    logger.warning(f"Data room still exists after cleanup: {resource_id}")
                    return False
            except Exception:
                # Exception likely means resource doesn't exist anymore
                logger.info(f"Verified cleanup succeeded for data_room: {resource_id} (exception)")
                return True

        elif resource_type == "session":
            # Sessions are harder to verify directly, assume cleanup worked
            return True

        else:
            logger.warning(f"Unknown resource type for verification: {resource_type}")
            return False

    except Exception:
        # Exception likely means resource doesn't exist anymore (successful cleanup)
        return True
